package com.lsw.captcha;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.support.annotation.IntDef;
import android.support.v7.app.AlertDialog;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.webkit.JsResult;
import android.webkit.WebSettings;
import android.webkit.WebView;

import com.lsw.captcha.web.LsWebChromeClient;
import com.lsw.captcha.web.LsWebViewClient;

import org.json.JSONException;
import org.json.JSONObject;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.HashMap;
import java.util.Map;

public class CaptchaUtil {
    private String WEB_HOST = "http://m.dev.lian-shong.cn/";
    private String mobile;
    private int type;
    private WebView webView;
    private AlertDialog alertDialog;
    private CaptchaCallback scriptInterface;

    //9-注册不登陆，0-找回密码、修改密码，1-登录、快速登录，4-验布登录
    public static final int USER_REGISTER = 9;
    public static final int USER_FOUND_PASSWORD = 0;
    public static final int USER_LOGIN = 1;
    public static final int CLOTH_REGISTER = 4;
    private Activity activity;

    @IntDef({USER_REGISTER, USER_FOUND_PASSWORD, USER_LOGIN, CLOTH_REGISTER})
    @Retention(RetentionPolicy.SOURCE)
    @interface AuthSliderType {
    }

    private CaptchaUtil() {
    }

    public static CaptchaUtil getNewInstance() {
        return new CaptchaUtil();
    }

    /***
     * 设置callback listener
     *
     * @param authSliderCallback {@link CaptchaCallback}
     * @return this
     */
    public CaptchaUtil setCaptchaCallback(CaptchaCallback authSliderCallback) {
        this.scriptInterface = authSliderCallback;
        return this;
    }

    /***
     * 创建WebView Dialog
     *
     * @param type 短信发送类型，9-注册不登陆，0-找回密码、修改密码，1-登录、快速登录
     * @return alertdialog
     */
    public CaptchaUtil setParameter(String mobile, @AuthSliderType int type) {
        this.mobile = mobile;
        this.type = type;
        return this;
    }

    @SuppressLint("AddJavascriptInterface")
    public CaptchaUtil show(Activity activity) {
        this.activity = activity;
        AlertDialog.Builder builder = new AlertDialog.Builder(activity, R.style.CaptchaDialog);
        alertDialog = builder.create();
        alertDialog.show();
        Window window = alertDialog.getWindow();
        if (window != null) {//调出dialog中的输入框使用的softInput
            window.clearFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE | WindowManager.LayoutParams.FLAG_ALT_FOCUSABLE_IM);
            window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
        }

        View view = LayoutInflater.from(activity).inflate(R.layout.captcha_view_layout, null, false);
        webView = (WebView) view.findViewById(R.id.sliderWebview);
        webView.setWebViewClient(new LsWebViewClient());
        webView.addJavascriptInterface(new LSJavaScriptInterface(new LSJavaScriptCallBackI() {
            @Override
            public void bridgeResponse(String action, String param) {
                parse(action, param);
            }
        }), "NativeInterface");
        webView.setWebChromeClient(new LsWebChromeClient() {
            @Override
            public boolean onJsAlert(WebView view, String url, String message, JsResult result) {
                return super.onJsAlert(view, url, message, result);
            }
        });
        alertDialog.setContentView(view);
        String url = WEB_HOST + "subject/service/sms.html?t=" + System.currentTimeMillis();
        WebSettings settings = webView.getSettings();
        String userAgent = settings.getUserAgentString();
        userAgent += " Application/" + "Headers";
        settings.setUserAgentString(userAgent);

        Map<String, String> headers = new HashMap<>();
        headers.put("token", "token");
        webView.loadUrl(url, headers);
        return this;
    }

    private void dismiss() {
        if (null != alertDialog) {
            alertDialog.dismiss();
        }
    }

    /***
     * 解析H5调用返回的信息
     *
     * @param action 行为
     * @param param  参数
     */
    private void parse(String action, String param) {
        switch (action) {
            case "methodGetSms":
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("mobile", mobile);
                    jsonObject.put("actType", type);
                    if (webView != null) {
                        webView.loadUrl("javascript:bridgeResponse('" + action + "','" + jsonObject.toString() + "')");
                    }
                    hideSoftInput();
                } catch (JSONException e) {
                    e.printStackTrace();
                }

                break;
            case "methodPushMessage":
                dismiss();
                if (!TextUtils.isEmpty(param) && null != scriptInterface) {
                    scriptInterface.onVerifyFail(param);
                }
                break;
            case "methodSmsSuccess":
                // 0已注册  1未注册
                try {
                    JSONObject jsonObject = new JSONObject(param);
                    String message = jsonObject.getString("message");
                    boolean isRegister = jsonObject.has("isRegister");
                    int typeStatus = 0;
                    if (isRegister) {
                        typeStatus = jsonObject.getInt("isRegister");
                    }
                    int status = jsonObject.getInt("status");
                    boolean wantRegister = typeStatus == 1;

                    if (null != scriptInterface) {
                        scriptInterface.onVerifySuccess(message, wantRegister, status);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                } finally {
                    dismiss();
                }
                break;
        }
    }

    /***
     * 隐藏软键盘
     */
    private void hideSoftInput() {
        InputMethodManager imm = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if (imm != null) {
            imm.hideSoftInputFromWindow(activity.getWindow().getDecorView().getWindowToken(), 0);
        }
    }

    public void onResume() {
        if (null != webView) {
            webView.onResume();
        }
    }

    public void onPause() {
        if (null != webView) {
            webView.onPause();
        }
    }

    public void onStop() {
        if (null != webView) {
            webView.stopLoading();
        }
    }

    public void onDestroy() {
        dismiss();
        if (null != webView) {
            webView.destroy();
            webView = null;
            alertDialog = null;
            scriptInterface = null;
        }
    }

    public interface CaptchaCallback {
        /**
         * 失败
         *
         * @param message 失败信息
         */
        void onVerifyFail(String message);

        /**
         * 验证成功
         *
         * @param message      成功信息
         * @param needRegister 是否已经注册
         * @param status       状态，暂时没用
         */
        void onVerifySuccess(String message, boolean needRegister, int status);
    }
}
