//
//  LSWebViewBridge.m
//  LSBuyer
//
//  Created by Li Zhiyu on 16/10/12.
//  Copyright © 2016年 lianshang. All rights reserved.
//

#import "LSWebViewBridge.h"
#import "NSString+LSConvert.h"
#import "UIView+LSNativeBradgeAction.h"

#define iOS8  [[UIDevice currentDevice]systemVersion].floatValue>=8.0

@interface LSWebViewBridge ()<UIWebViewDelegate,WKNavigationDelegate, WKUIDelegate,WKScriptMessageHandler>

@property (strong, nonatomic) UIWebView *mainWebView;

//js与native交互
@property(nonatomic,retain) WKUserContentController *wkJsController;

@property (nonatomic, strong) UIProgressView *progressView;

@end

@implementation LSWebViewBridge

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if(self) {
        self.autoresizingMask=UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
        // 添加进入条
        self.progressView = [[UIProgressView alloc] init];
        self.progressView.frame = self.bounds;
        self.progressView.autoresizingMask=UIViewAutoresizingFlexibleWidth;
        self.progressView.progressTintColor=[UIColor redColor];
        self.progressView.trackTintColor=[UIColor clearColor];
        self.progressView.backgroundColor = [UIColor redColor];
        if(iOS8){
            [self addSubview:self.wkWebView];
            [self addSubview:self.progressView];
        }else{
            [self addSubview:self.mainWebView];
            self.mainWebView.delegate=self;
        }
    }
    return self;
}

-(void)setUrl:(NSString *)url{
    _url=url;
    if(![_url hasPrefix:@"http"]){
        _url = [NSString stringWithFormat:@"http://%@",_url];
    }
    self.currentUrl = _url;
    [self loadUrl:self.url];
}

-(void)loadUrl:(NSString*)url{
    NSMutableURLRequest *appReq = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
    
    [self loadURLRequest:appReq];
}

-(void)loadURLRequest:(NSMutableURLRequest*)request{
    NSMutableURLRequest *req = [self setRequestHeader:request];
    if(req){
        request = req;
    }
    if(iOS8){
        [self.wkWebView performSelector:@selector(loadRequest:) withObject:request afterDelay:0];
    }else{
        //    self.mainWebView.hidden=NO;
        [self.mainWebView loadRequest:request];
    }
}

-(NSMutableURLRequest*)setRequestHeader:(NSMutableURLRequest*)request{
    
    if(request){
        
        NSDictionary *dict = request.allHTTPHeaderFields;
        
        NSLog(@"urlheader:%@",dict);
        
        if(![self isContentHeader:request]){
            [request setValue:@"lsbuyer" forHTTPHeaderField:@"appName"];
            [request setValue:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"] forHTTPHeaderField:@"appVersion"];
            
        }
        return request;
        
    }
    return nil;
}

- (BOOL)isContentHeader:(NSMutableURLRequest*)request{
    NSDictionary *dict = request.allHTTPHeaderFields;
    
//    NSLog(@"urlheader:%@",dict);
    
//    if(![dict containsObjectForKey:@"appName"] || ![dict containsObjectForKey:@"appVersion"]){
//        return NO;
//    }
    
    if(![[dict allKeys] containsObject:@"appName"] || ![[dict allKeys] containsObject:@"appVersion"]){
        return NO;
    }

    return YES;
}

#pragma -mark UIWebView delegate

-(BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType{
    NSString *requestString = [[[request URL] absoluteString] stringByReplacingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
    NSLog(@"requestString:%@",requestString);
    //http://www.baid.com/?aa=bb&kkk=ooo
    NSURL *urls=request.URL;
    NSString *scheme=urls.scheme;//http
    NSString *host=urls.host;//www.baid.com

    NSMutableURLRequest * appReq = (NSMutableURLRequest *)request;
//    [mRequest setValue:@"lsbuyer" forHTTPHeaderField:@"appName"];
//    [mRequest setValue:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"] forHTTPHeaderField:@"appVersion"];
    NSMutableURLRequest *req = [self setRequestHeader:appReq];
    if(req && [requestString hasPrefix:@"http"] && ([requestString rangeOfString:@"lianshang.com"].location !=NSNotFound || [requestString rangeOfString:@"lian-shang.cn"].location !=NSNotFound)){
        [self loadURLRequest:req];
        return NO;
    }
    
    //参数
    NSString *query=[urls.query stringByReplacingPercentEscapesUsingEncoding: NSUTF8StringEncoding];//aa=bb&kkk=ooo
    if([scheme isEqualToString:@"lsjscallnative"]){
        
        BOOL isExed = [self.mainWebView nativeAction:host param:query];
        if(!isExed){
            if(_delegate&&[_delegate respondsToSelector:@selector(jsCallNativeAction:param:)]){
                [_delegate jsCallNativeAction:host param:query];
            }
        }
        
        
        return NO;
    }
    
    return YES;
}

-(void)webViewDidFinishLoad:(UIWebView *)webView{
    
    NSString *title=[self.mainWebView stringByEvaluatingJavaScriptFromString:@"document.title"];//获取当前页面的title
    if(_delegate&&[_delegate respondsToSelector:@selector(htmlTitle:)]){
        [_delegate htmlTitle:title];
    }
}

#pragma -mark WKWebView
-(WKWebView *)wkWebView{
    if(!_wkWebView){
        
        WKWebViewConfiguration *config = [[WKWebViewConfiguration alloc] init];
        // 设置偏好设置
        config.preferences = [[WKPreferences alloc] init];
        // 默认为0
        config.preferences.minimumFontSize = 10;
        // 默认认为YES
        config.preferences.javaScriptEnabled = YES;
        // 在iOS上默认为NO，表示不能js自动通过窗口打开，需要通过用户交互才能打开窗口
        config.preferences.javaScriptCanOpenWindowsAutomatically = NO;
        config.userContentController=self.wkJsController;
        
        _wkWebView=[[WKWebView alloc] initWithFrame:self.bounds configuration:config];
        _wkWebView.autoresizingMask=UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
        // 导航代理
        _wkWebView.navigationDelegate = self;
        // 与webview UI交互代理
        _wkWebView.UIDelegate = self;
        
        // 添加KVO监听
        [_wkWebView addObserver:self
                     forKeyPath:@"loading"
                        options:NSKeyValueObservingOptionNew
                        context:nil];
        [_wkWebView addObserver:self
                     forKeyPath:@"title"
                        options:NSKeyValueObservingOptionNew
                        context:nil];
        [_wkWebView addObserver:self
                     forKeyPath:@"estimatedProgress"
                        options:NSKeyValueObservingOptionNew
                        context:nil];
        // 配置agent
        [_wkWebView evaluateJavaScript:@"navigator.userAgent" completionHandler:^(id result, NSError *error) {
            NSString *userAgent = result;
            NSString *newUserAgent = [userAgent stringByAppendingString:@"/LSBUYER/"];
            NSString *appVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
            newUserAgent = [newUserAgent stringByAppendingString:appVersion];
            NSDictionary *dictionary = [NSDictionary dictionaryWithObjectsAndKeys:newUserAgent, @"UserAgent", nil];
            [[NSUserDefaults standardUserDefaults] registerDefaults:dictionary];
            [[NSUserDefaults standardUserDefaults] synchronize];
            
            // 适配iOS 9
            if ([[UIDevice currentDevice].systemVersion floatValue] >= 9.0) {
                [_wkWebView setCustomUserAgent:newUserAgent];
            }
            [_wkWebView evaluateJavaScript:@"navigator.userAgent" completionHandler:^(id result, NSError * _Nullable error) {
                NSLog(@"----%@",result);
            }];
        }];
    }
    return _wkWebView;
}

-(UIWebView *)mainWebView{
    if(!_mainWebView){
        _mainWebView=[[UIWebView alloc] initWithFrame:self.bounds];
        _mainWebView.delegate=self;
        _mainWebView.autoresizingMask=UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
    }
    return _mainWebView;
}

-(WKUserContentController *)wkJsController{
    if(!_wkJsController){
        _wkJsController=[[WKUserContentController alloc] init];
        [_wkJsController addScriptMessageHandler:self name:@"CallNativeIOS"];
        //添加支持的native call API
        //        NSArray *actions=@[@"login",@"alert"];
        //        for(NSString *action in actions){
        //            [_wkJsController addScriptMessageHandler:self name:action];
        //        }
        
    }
    return _wkJsController;
}

///**
// 获取H5 定制的事件
//
// @param userContentController <#userContentController description#>
// @param message <#message description#>
// */
//-(void)userContentController:(WKUserContentController *)userContentController didReceiveScriptMessage:(WKScriptMessage *)message{
//    if([message.name isEqualToString:@"CallNativeIOS"]){
//        NSLog(@"message.body:%@",message.body);
//        NSDictionary *body=message.body;
//        NSString *action=body[@"action"];
//        NSString *param=body[@"param"];
//        // 处理金融的 H5 页面title
//        if ([action isEqualToString:@"efChangeTitle"]) {
//            NSDictionary *dic = [NSString dictionaryWithJsonString:body[@"param"]];
//            NSLog(@"----%@",dic[@"title"]);
//            if(_delegate&&[_delegate respondsToSelector:@selector(htmlTitle:)]){
//                [_delegate htmlTitle:dic[@"title"]];
//            }
//        }
//        BOOL isExed = [self.wkWebView nativeAction:action param:param];
//        if(!isExed){
//            if(_delegate&&[_delegate respondsToSelector:@selector(jsCallNativeAction:param:)]){
//                [_delegate jsCallNativeAction:action param:param];
//            }
//        }
//    }
//}

- (void)observeValueForKeyPath:(NSString *)keyPath
                      ofObject:(id)object
                        change:(NSDictionary<NSString *,id> *)change
                       context:(void *)context {
    if ([keyPath isEqualToString:@"loading"]) {
        NSLog(@"loading");
    } else if ([keyPath isEqualToString:@"title"]) {
        if (object == self.wkWebView) {
            NSLog(@"-----++++%@", self.wkWebView.title);
        }
        else {
            [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
        }
        if(_delegate&&[_delegate respondsToSelector:@selector(htmlTitle:)]){
            [_delegate htmlTitle:self.wkWebView.title];
        }
    } else if ([keyPath isEqualToString:@"estimatedProgress"]) {
        NSLog(@"progress: %f", self.wkWebView.estimatedProgress);
        self.progressView.progress = self.wkWebView.estimatedProgress;
    }
}

#pragma mark - WKNavigationDelegate
- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
    NSString *url = navigationAction.request.URL.absoluteString;
    NSMutableURLRequest * appReq = (NSMutableURLRequest *)navigationAction.request;
    self.currentUrl = url;
//    [mRequest setValue:@"lsbuyer" forHTTPHeaderField:@"appName"];
//    [mRequest setValue:[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"] forHTTPHeaderField:@"appVersion"];
    if(![self isContentHeader:appReq] && [url hasPrefix:@"http"] && ([url rangeOfString:@"lianshang.com"].location !=NSNotFound || [url rangeOfString:@"lian-shang.cn"].location !=NSNotFound)){
        
        [self loadURLRequest:appReq];
        
        decisionHandler(WKNavigationActionPolicyCancel);
        
        
        return;
    }
    
//    NSString *hostname = navigationAction.request.URL.host.lowercaseString;
    if (navigationAction.navigationType == WKNavigationTypeLinkActivated) {
        if([url hasPrefix:@"tel:"]){
            url = [url stringByReplacingOccurrencesOfString:@"tel:" withString:@"telprompt://"];
            
        }
        if([url hasPrefix:@"http"]){
            decisionHandler(WKNavigationActionPolicyAllow);
        }else{
            [[UIApplication sharedApplication] openURL:[NSURL URLWithString:url]];
            decisionHandler(WKNavigationActionPolicyCancel);
        }
        
        
    } else {
    
        decisionHandler(WKNavigationActionPolicyAllow);
    }
    
    NSLog(@"%s", __FUNCTION__);
}

- (void)webView:(WKWebView *)webView decidePolicyForNavigationResponse:(WKNavigationResponse *)navigationResponse decisionHandler:(void (^)(WKNavigationResponsePolicy))decisionHandler {
    decisionHandler(WKNavigationResponsePolicyAllow);
    NSLog(@"%s-----code:%li", __FUNCTION__,((NSHTTPURLResponse *)navigationResponse.response).statusCode);
}

- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(null_unspecified WKNavigation *)navigation {
    NSLog(@"%s", __FUNCTION__);
    self.progressView.alpha = 1.0;
}

- (void)webView:(WKWebView *)webView didReceiveServerRedirectForProvisionalNavigation:(null_unspecified WKNavigation *)navigation {
    NSLog(@"%s", __FUNCTION__);
}

- (void)webView:(WKWebView *)webView didFailProvisionalNavigation:(null_unspecified WKNavigation *)navigation withError:(NSError *)error {
    NSLog(@"%s", __FUNCTION__);
    self.progressView.alpha = 0;
}

- (void)webView:(WKWebView *)webView didCommitNavigation:(null_unspecified WKNavigation *)navigation {
    NSLog(@"%s", __FUNCTION__);
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(null_unspecified WKNavigation *)navigation {
    NSLog(@"%s", __FUNCTION__);
    [UIView animateWithDuration:0.5 animations:^{
        self.progressView.alpha = 0;
    }];
}

- (void)webView:(WKWebView *)webView didFailNavigation:(null_unspecified WKNavigation *)navigation withError:(NSError *)error {
    
}

- (void)webView:(WKWebView *)webView didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential *__nullable credential))completionHandler {
    NSLog(@"%s", __FUNCTION__);
    completionHandler(NSURLSessionAuthChallengePerformDefaultHandling, nil);
}

- (void)webViewWebContentProcessDidTerminate:(WKWebView *)webView {
    NSLog(@"%s", __FUNCTION__);
}

#pragma mark - WKUIDelegate
- (void)webViewDidClose:(WKWebView *)webView {
    NSLog(@"%s", __FUNCTION__);
}

- (void)webView:(WKWebView *)webView runJavaScriptAlertPanelWithMessage:(NSString *)message initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(void))completionHandler {
    NSLog(@"%s", __FUNCTION__);
//    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:message preferredStyle:UIAlertControllerStyleAlert];
//    [alert addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
//        completionHandler();
//    }]];
//
//    [[XYRouter currentNavigationController] presentViewController:alert animated:YES completion:NULL];
    NSLog(@"%@", message);
}

- (void)webView:(WKWebView *)webView runJavaScriptConfirmPanelWithMessage:(NSString *)message initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(BOOL result))completionHandler {
    NSLog(@"%s", __FUNCTION__);
    
//    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"JS调用confirm" preferredStyle:UIAlertControllerStyleAlert];
//    [alert addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
//        completionHandler(YES);
//    }]];
//    [alert addAction:[UIAlertAction actionWithTitle:@"取消" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
//        completionHandler(NO);
//    }]];
//    [[XYRouter currentNavigationController] presentViewController:alert animated:YES completion:NULL];
    
    NSLog(@"%@", message);
}

- (void)webView:(WKWebView *)webView runJavaScriptTextInputPanelWithPrompt:(NSString *)prompt defaultText:(nullable NSString *)defaultText initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(NSString * __nullable result))completionHandler {
    NSLog(@"%s", __FUNCTION__);
    
    NSLog(@"%@", prompt);
//    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:prompt preferredStyle:UIAlertControllerStyleAlert];
//    [alert addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
//        //        textField.textColor = [UIColor redColor];
//        textField.text=defaultText;
//    }];
//
//    [alert addAction:[UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
//        completionHandler([[alert.textFields lastObject] text]);
//    }]];
//
//    [[XYRouter currentNavigationController] presentViewController:alert animated:YES completion:NULL];
}


-(void)callJavaScript:(NSString*)func responseBlock:(NativeCallJSResponse _Null_unspecified)responseBlock{
    if(iOS8){
        [self.wkWebView callFunction:func responseBlock:responseBlock];
    }else{
        [self.mainWebView callFunction:func responseBlock:responseBlock];
    }
}

-(void)callJavaScriptBridgeResponse:(NSString *)action param:(NSString *)param responseBlock:(NativeCallJSResponse)responseBlock{
    if(iOS8){
        [self.wkWebView callJSResponseAction:action param:param responseBlock:responseBlock];
    }else{
        [self.mainWebView callJSResponseAction:action param:param responseBlock:responseBlock];
    }
}




-(void)setDelegate:(id<LSWebViewBridgeDelegate>)delegate{
    _delegate = delegate;
    
//    if(iOS8){
//        
//        if(_delegate){
//            self.wkWebView.navigationDelegate = self;
//            self.wkWebView.UIDelegate = self;
//        }else{
//            self.wkWebView.navigationDelegate = nil;
//            self.wkWebView.UIDelegate = nil;
//        }
//    }else{
//        //    self.mainWebView.hidden=NO;
//        if(_delegate){
//            self.mainWebView.delegate = self;
//        }else{
//            self.mainWebView.delegate = nil;
//        }
//    }
}


@end
